@@ -5,11 +5,13 @@
  */
 
 #include <common.h>
+#include <init.h>
 #include <asm/io.h>
 #include <dm.h>
 #include <linux/bitfield.h>
 #include <regmap.h>
 #include <syscon.h>
+#include <linux/bitops.h>
 #include <linux/err.h>
 
 #define AO_SEC_SD_CFG8		0xe0
@@ -36,6 +38,7 @@ static const struct meson_gx_soc_id {
 	{ "G12A",   0x28 },
 	{ "G12B",   0x29 },
 	{ "SM1",    0x2b },
+	{ "A1",	    0x2c },
 };
 
 static const struct meson_gx_package_id {
@@ -63,7 +66,11 @@ static const struct meson_gx_package_id {
 	{ "S905X2", 0x28, 0x40, 0xf0 },
 	{ "A311D",  0x29, 0x10, 0xf0 },
 	{ "S922X",  0x29, 0x40, 0xf0 },
-	{ "S905X3", 0x2b, 0x5, 0xf },
+	{ "S905D3", 0x2b, 0x4, 0xf5 },
+	{ "S905X3", 0x2b, 0x5, 0xf5 },
+	{ "S905X3", 0x2b, 0x10, 0x3f },
+	{ "S905D3", 0x2b, 0x30, 0x3f },
+	{ "A113L", 0x2c, 0x0, 0xf8 },
 };
 
 DECLARE_GLOBAL_DATA_PTR;
