@@ -6,6 +6,8 @@
 
 /* CPU specific code */
 #include <common.h>
+#include <cpu_func.h>
+#include <irq_func.h>
 #include <asm/cache.h>
 
 /*
