@@ -1,9 +1,8 @@
+/* SPDX-License-Identifier: BSD-3-Clause */
 /*
  * This file is part of the libpayload project.
  *
  * Copyright (C) 2008 Advanced Micro Devices, Inc.
- *
- * SPDX-License-Identifier:	BSD-3-Clause
  */
 
 #ifndef _COREBOOT_SYSINFO_H
@@ -50,6 +49,8 @@ struct sysinfo_t {
 	u32	vdat_size;
 	void	*tstamp_table;
 	void	*cbmem_cons;
+	u64 smbios_start;
+	u32 smbios_size;
 
 	struct cb_serial *serial;
 };
