@@ -5,7 +5,9 @@
 
 #include <common.h>
 #include <dm.h>
-#include <environment.h>
+#include <env.h>
+#include <init.h>
+#include <net.h>
 #include <asm/io.h>
 #include <asm/arch/gx.h>
 #include <asm/arch/sm.h>
