@@ -6,7 +6,8 @@
 
 #include <common.h>
 #include <dm.h>
-#include <environment.h>
+#include <init.h>
+#include <net.h>
 #include <asm/io.h>
 #include <asm/arch/axg.h>
 #include <asm/arch/sm.h>
@@ -17,5 +18,7 @@ int misc_init_r(void)
 {
 	meson_eth_init(PHY_INTERFACE_MODE_RGMII, 0);
 
+	meson_generate_serial_ethaddr();
+
 	return 0;
 }
