@@ -356,7 +356,9 @@ int stdio_add_devices(void)
 	} else {
 		if (IS_ENABLED(CONFIG_LCD))
 			drv_lcd_init();
-		if (IS_ENABLED(CONFIG_VIDEO) || IS_ENABLED(CONFIG_CFB_CONSOLE))
+		if (IS_ENABLED(CONFIG_VIDEO) ||
+		    IS_ENABLED(CONFIG_CFB_CONSOLE) ||
+		    IS_ENABLED(CONFIG_VIDEO_VCXK))
 			drv_video_init();
 	}
 
