@@ -1,16 +1,19 @@
+// SPDX-License-Identifier: GPL-2.0+
 /*
  * (C) Copyright 2000-2011
  * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
- *
- * SPDX-License-Identifier:	GPL-2.0+
  */
 
 #include <common.h>
 #include <ata.h>
+#include <blk.h>
 #include <dm.h>
 #include <ide.h>
+#include <log.h>
+#include <part.h>
 #include <watchdog.h>
 #include <asm/io.h>
+#include <linux/delay.h>
 
 #ifdef __PPC__
 # define EIEIO		__asm__ volatile ("eieio")
@@ -232,7 +235,7 @@ unsigned char atapi_issue(int device, unsigned char *ccb, int ccblen,
 		 (unsigned char) ((buflen >> 8) & 0xFF));
 	ide_outb(device, ATA_DEV_HD, ATA_LBA | ATA_DEVICE(device));
 
-	ide_outb(device, ATA_COMMAND, ATAPI_CMD_PACKET);
+	ide_outb(device, ATA_COMMAND, ATA_CMD_PACKET);
 	udelay(50);
 
 	mask = ATA_STAT_DRQ | ATA_STAT_BUSY | ATA_STAT_ERR;
@@ -571,7 +574,7 @@ static void ide_ident(struct blk_desc *dev_desc)
 			/*
 			 * Start Ident Command
 			 */
-			ide_outb(device, ATA_COMMAND, ATAPI_CMD_IDENT);
+			ide_outb(device, ATA_COMMAND, ATA_CMD_ID_ATAPI);
 			/*
 			 * Wait for completion - ATAPI devices need more time
 			 * to become ready
@@ -583,7 +586,7 @@ static void ide_ident(struct blk_desc *dev_desc)
 			/*
 			 * Start Ident Command
 			 */
-			ide_outb(device, ATA_COMMAND, ATA_CMD_IDENT);
+			ide_outb(device, ATA_COMMAND, ATA_CMD_ID_ATA);
 
 			/*
 			 * Wait for completion
@@ -967,7 +970,7 @@ ulong ide_read(struct blk_desc *block_dev, lbaint_t blknr, lbaint_t blkcnt,
 
 	/* first check if the drive is in Powersaving mode, if yes,
 	 * increase the timeout value */
-	ide_outb(device, ATA_COMMAND, ATA_CMD_CHK_PWR);
+	ide_outb(device, ATA_COMMAND, ATA_CMD_CHK_POWER);
 	udelay(50);
 
 	c = ide_wait(device, IDE_TIME_OUT);	/* can't take over 500 ms */
@@ -1170,8 +1173,6 @@ static int ide_blk_probe(struct udevice *udev)
 		BLK_REV_SIZE);
 	desc->revision[BLK_REV_SIZE] = '\0';
 
-	part_init(desc);
-
 	return 0;
 }
 
