@@ -55,8 +55,6 @@
 
 #define CONFIG_HIKEY_GPIO
 
-/* Command line configuration */
-
 /* BOOTP options */
 #define CONFIG_BOOTP_BOOTFILESIZE
 
@@ -82,9 +80,6 @@
 				BOOTENV
 
 /* Preserve environment on eMMC */
-#define CONFIG_ENV_SIZE			0x1000
-#define CONFIG_SYS_MMC_ENV_DEV		0	/* Use eMMC */
-#define CONFIG_SYS_MMC_ENV_PART		2	/* Use Boot1 partition */
 
 /* Monitor Command Prompt */
 #define CONFIG_SYS_CBSIZE		512	/* Console I/O Buffer Size */
